@@ -2,6 +2,7 @@
  * Seahorse
  *
  * Copyright (C) 2003 Jacob Perkins
+ * Copyright (C) 2004-2005 Nate Nielsen
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -23,49 +24,24 @@
 #include <libintl.h>
 #include <gnome.h>
 
+#include "seahorse-operation.h"
+#include "seahorse-progress.h"
 #include "seahorse-libdialogs.h"
 #include "seahorse-widget.h"
 #include "seahorse-validity.h"
 #include "seahorse-recipients-store.h"
+#include "seahorse-default-key-control.h"
+#include "seahorse-gconf.h"
 
 #define VIEW "keys"
 #define OK "ok"
 
-static void
-show_progress (SeahorseContext *sctx, const gchar *op, gdouble fract, SeahorseWidget *swidget)
-{
-	GnomeAppBar *status;
-	GtkProgressBar *progress;
-	gboolean sensitive;
-
-	sensitive = (fract == -1);
-	
-	status = GNOME_APPBAR (glade_xml_get_widget (swidget->xml, "status"));
-	gnome_appbar_set_status (status, op);
-	progress = gnome_appbar_get_progress (status);
-	/* do progress */
-	if (fract <= 1 && fract > 0)
-		gtk_progress_bar_set_fraction (progress, fract);
-	else if (fract != -1) {
-		gtk_progress_bar_set_pulse_step (progress, 0.05);
-		gtk_progress_bar_pulse (progress);
-	}
-	/* if fract == -1, cleanup progress */
-	else
-		gtk_progress_bar_set_fraction (progress, 0);
-	
-	gtk_widget_set_sensitive (glade_xml_get_widget (swidget->xml, OK), sensitive);
-	
-	while (g_main_context_pending (NULL))
-		g_main_context_iteration (NULL, TRUE);
-}
-
 static void
 selection_changed (GtkTreeSelection *selection, SeahorseWidget *swidget)
 {
 	GList *list = NULL, *keys = NULL;
 	gint selected = 0, invalid = 0;
-	gchar* msg;
+	gchar *msg, *s1, *s2;
    
 	list = seahorse_key_store_get_selected_keys (GTK_TREE_VIEW (
 		glade_xml_get_widget (swidget->xml, VIEW)));
@@ -78,16 +54,32 @@ selection_changed (GtkTreeSelection *selection, SeahorseWidget *swidget)
  
     if(invalid == 0) {
         msg = g_strdup_printf(
-            ngettext("Selected %d keys", "Selected %d keys", selected), selected);
+            ngettext("Selected %d recipient", "Selected %d recipients", selected), selected);
             
     /* For translators */
-    } else if (invalid == 1) {
-        msg = g_strdup_printf (
-            ngettext("Selected %d not fully valid key", "Selected %d keys (%d not fully valid)", selected), selected, invalid);
-            
+    } else if (invalid == selected) {
+        msg = g_strdup_printf (ngettext("Selected %d not fully valid recipient", 
+                                        "Selected %d not fully valid recipients", selected), 
+                               selected);
+
     } else {
-        msg = g_strdup_printf (
-            ngettext("Selected %d not fully valid key", "Selected %d keys (%d not fully valid)", selected), selected, invalid);
+       
+        /* TRANSLATOR: This string will become
+          * "Selected %d recipients (%d not fully valid)" */
+        s1 = g_strdup_printf(ngettext("Selected %d recipient ", "Selected %d recipients ", selected),
+                             selected);
+
+        /* TRANSLATOR: This string will become
+         * "Selected %d recipients (%d not fully valid)" */
+        s2 = g_strdup_printf(ngettext("(%d not fully valid)", "(%d not fully valid)", invalid),
+                             invalid);
+
+        /* TRANSLATOR: "%s%s" are "Selected %d recipients (%d not fully valid)"
+         * Swap order with "%2$s%1$s" if needed */
+        msg = g_strdup_printf(_("%s%s"), s1, s2);  
+        
+        g_free (s1);
+        g_free (s2);     
     }
         
 	gnome_appbar_set_status (GNOME_APPBAR (glade_xml_get_widget (swidget->xml, "status")), msg);
@@ -115,6 +107,12 @@ filter_changed (GtkWidget *widget, SeahorseKeyStore* skstore)
     g_object_set (skstore, "filter", text, NULL);
 }
 
+static void
+filter_activated (GtkEntry *entry, GtkWidget *widget)
+{
+    gtk_widget_grab_focus (widget);
+}
+
 /* Called when properties on the SeahorseKeyStore object change */
 static void
 update_filters (GObject* object, GParamSpec* arg, SeahorseWidget* swidget)
@@ -136,39 +134,73 @@ update_filters (GObject* object, GParamSpec* arg, SeahorseWidget* swidget)
 }
 
 GList*
-seahorse_recipients_get (SeahorseContext *sctx)
+seahorse_recipients_get (SeahorseContext *sctx, SeahorseKeyPair **signer)
 {
 	SeahorseWidget *swidget;
+    SeahorseOperation *operation;
+    SeahorseDefaultKeyControl *sdkc;
 	GtkTreeSelection *selection;
 	GtkTreeView *view;
 	GtkWidget *widget;
 	gint response;
 	gboolean done = FALSE;
     GList *keys = NULL;
+    gchar *id;
     SeahorseKeyStore* skstore;
+    SeahorseKeySource *sksrc;
 	
 	swidget = seahorse_widget_new ("recipients", sctx);
 	g_return_val_if_fail (swidget != NULL, NULL);
 	
-	g_signal_connect (swidget->sctx, "progress", G_CALLBACK (show_progress), swidget);
-	
 	view = GTK_TREE_VIEW (glade_xml_get_widget (swidget->xml, VIEW));
 	selection = gtk_tree_view_get_selection (view);
 	gtk_tree_selection_set_mode (selection, GTK_SELECTION_SINGLE);
 	g_signal_connect (selection, "changed",
 		G_CALLBACK (selection_changed), swidget);
         
-	skstore = seahorse_recipients_store_new (sctx, view);
+    sksrc = seahorse_context_get_key_source (sctx);
+    g_return_val_if_fail (sksrc != NULL, NULL);
+
+    /* Hook progress bar in */
+    operation = seahorse_key_source_get_operation (sksrc);
+    g_return_val_if_fail (operation != NULL, NULL);
+
+    /* If always using the default key for signing, then hide this section */
+    if (!signer || (*signer = seahorse_context_get_default_key (sctx)) != NULL) {
+        widget = glade_xml_get_widget (swidget->xml, "sign_box");
+        gtk_widget_hide (widget);
+
+    /* Signing section */
+    } else {
+        widget = glade_xml_get_widget (swidget->xml, "sign_key_place");
+        sdkc = seahorse_default_key_control_new (sksrc, _("None (Don't sign)"));
+        gtk_container_add (GTK_CONTAINER (widget), GTK_WIDGET (sdkc));
+        gtk_widget_show_all (widget);    
+
+        /* Select the last key used */
+        id = seahorse_gconf_get_string (LASTSIGNER_KEY);
+        seahorse_default_key_control_select_id (sdkc, id);
+        g_free (id); 
+    }
+    
+    widget = glade_xml_get_widget (swidget->xml, "status");
+    seahorse_progress_appbar_set_operation (widget, operation);
+        
+	skstore = seahorse_recipients_store_new (sksrc, view);
    
     glade_xml_signal_connect_data (swidget->xml, "on_mode_changed", 
                               G_CALLBACK (mode_changed), skstore);
     glade_xml_signal_connect_data (swidget->xml, "on_filter_changed",
                               G_CALLBACK (filter_changed), skstore);
+    glade_xml_signal_connect_data (swidget->xml, "on_filter_activate",
+                              G_CALLBACK (filter_activated), view);
 
     g_signal_connect (skstore, "notify", G_CALLBACK (update_filters), swidget);
     update_filters (G_OBJECT (skstore), NULL, swidget);
 
-	widget = glade_xml_get_widget (swidget->xml, swidget->name);
+	widget = seahorse_widget_get_top (swidget);
+    seahorse_widget_show (swidget);
+    
 	while (!done) {
 		response = gtk_dialog_run (GTK_DIALOG (widget));
 		switch (response) {
@@ -181,8 +213,16 @@ seahorse_recipients_get (SeahorseContext *sctx)
 				break;
 		}
 	}
-	
-    g_signal_handlers_disconnect_by_func (swidget->sctx, G_CALLBACK (show_progress), swidget);	
+
+    if (keys && signer) {
+        if (!*signer) 
+            *signer = seahorse_default_key_control_active (sdkc);
+
+        /* Save this as the last key signed with */
+        seahorse_gconf_set_string (LASTSIGNER_KEY, *signer == NULL ? 
+                        "" : seahorse_key_pair_get_id (*signer));
+    }
+    
 	seahorse_widget_destroy (swidget);
 	return keys;
 }
