@@ -21,6 +21,7 @@
  
 #include <gnome.h>
  
+#include "seahorse-gpgmex.h"
 #include "seahorse-key-dialogs.h"
 #include "seahorse-key-widget.h"
 #include "seahorse-key-op.h"
@@ -77,7 +78,7 @@ ok_clicked (GtkButton *button, SeahorseWidget *swidget)
 	err = seahorse_key_pair_op_add_uid (SEAHORSE_KEY_PAIR (skey),
 		                                name, email, comment);
 	if (!GPG_IS_OK (err))
-		seahorse_util_handle_error (err, _("Couldn't add user id"));
+		seahorse_util_handle_gpgme (err, _("Couldn't add user id"));
 	else
 		seahorse_widget_destroy (swidget);
 }
